import pandas as pd
import msgpack
import json

def main():
    # load data
    df_opencc_s2t = pd.read_excel('./tables/opencc_s2t.xlsx', header=None)
    df_opencc_t2s = pd.read_excel('./tables/opencc_t2s.xlsx', header=None)
    df_variants = pd.read_excel('./tables/variants.xlsx', sheet_name=0)

    variants = df_variants['yitizi'].tolist()

    df_opencc_s2t_filtered = df_opencc_s2t[~df_opencc_s2t[0].isin(variants)]
    df_opencc_t2s_filtered = df_opencc_t2s[~df_opencc_t2s[0].isin(variants)]

    df_opencc_s2t_new = pd.DataFrame(columns=['char_before', 'char_after', 'source'])
    columns = df_opencc_s2t_filtered.columns
    for i in df_opencc_s2t_filtered.index:
        yitizi = df_opencc_s2t_filtered.loc[i, 0]
        t_list = df_opencc_s2t_filtered.loc[i, 1:].dropna().tolist()
        t_str = '|'.join(t_list)
        new_row = {'char_before': yitizi, 'char_after': t_str, 'source': 'opencc_s2t'}
        df_opencc_s2t_new.loc[len(df_opencc_s2t_new)] = new_row

    df_opencc_t2s_new = pd.DataFrame(columns=['char_before', 'char_after', 'source'])
    columns = df_opencc_t2s_filtered.columns
    for i in df_opencc_t2s_filtered.index:
        yitizi = df_opencc_t2s_filtered.loc[i, 0]
        s_list = df_opencc_t2s_filtered.loc[i, 1:].dropna().tolist()
        s_str = '|'.join(s_list)
        new_row = {'char_before': yitizi, 'char_after': s_str, 'source': 'opencc_t2s'}
        df_opencc_t2s_new.loc[len(df_opencc_t2s_new)] = new_row

    # generate v2t and v2s
    df_variants_v2t = df_variants[['yitizi', 'zhengzi_t']]
    df_variants_v2t = df_variants_v2t.copy()
    df_variants_v2s = df_variants[['yitizi', 'zhengzi_s']]
    df_variants_v2s = df_variants_v2s.copy()

    df_variants_v2t_new = pd.DataFrame(columns=['char_before', 'char_after', 'source'])
    for variant in df_variants_v2t['yitizi'].unique():
        zhengzi_t_list = df_variants_v2t[df_variants_v2t['yitizi']==variant]['zhengzi_t'].tolist()
        zhengzi_t_str = '|'.join(zhengzi_t_list)
        new_row = {'char_before': variant, 'char_after': zhengzi_t_str, 'source': 'variants'}
        df_variants_v2t_new.loc[len(df_variants_v2t_new)] = new_row

    df_variants_v2s_new = pd.DataFrame(columns=['char_before', 'char_after', 'source'])
    for variant in df_variants_v2s['yitizi'].unique():
        zhengzi_s_list = df_variants_v2s[df_variants_v2s['yitizi']==variant]['zhengzi_s'].tolist()
        zhengzi_s_str = '|'.join(zhengzi_s_list)
        new_row = {'char_before': variant, 'char_after': zhengzi_s_str, 'source': 'variants'}
        df_variants_v2s_new.loc[len(df_variants_v2s_new)] = new_row

    df_variants_v2t_final = pd.concat([df_variants_v2t_new, df_opencc_s2t_new], ignore_index=True)
    df_variants_v2s_final = pd.concat([df_variants_v2s_new, df_opencc_t2s_new], ignore_index=True)

    df_variants_v2t_final.to_excel('./tables/variants_v2t.xlsx', index=False)
    df_variants_v2s_final.to_excel('./tables/variants_v2s.xlsx', index=False)


    # load dataframe
    df_v2t = pd.read_excel('./tables/variants_v2t.xlsx', sheet_name=0)
    df_v2s = pd.read_excel('./tables/variants_v2s.xlsx', sheet_name=0)

    # convert dataframe to dict
    dict_v2t = {}
    for i in df_v2t.index:
        source = df_v2t.loc[i, 'source']
        char_before = df_v2t.loc[i, 'char_before']
        char_after = df_v2t.loc[i, 'char_after']
        char_after_list = char_after.split('|')
        dict_v2t[char_before] = char_after_list

    dict_v2s = {}
    for i in df_v2s.index:
        source = df_v2s.loc[i, 'source']
        char_before = df_v2s.loc[i, 'char_before']
        char_after = df_v2s.loc[i, 'char_after']
        char_after_list = char_after.split('|')
        dict_v2s[char_before] = char_after_list

    # save dict as json
    with open("./data/v2t.json", "w") as json_file:
        json.dump(dict_v2t, json_file)
    with open("./data/v2s.json", "w") as json_file:
        json.dump(dict_v2s, json_file)

    # convert dict to MessagePack and save
    msgpack_file = './data/v2t.msgpack'
    data = dict_v2t
    with open(msgpack_file, 'wb') as f:
        packed_data = msgpack.packb(data, use_bin_type=True)
        f.write(packed_data)
    msgpack_file = './data/v2s.msgpack'
    data = dict_v2s
    with open(msgpack_file, 'wb') as f:
        packed_data = msgpack.packb(data, use_bin_type=True)
        f.write(packed_data)

if __name__ == '__main__':
    main()


import os
import msgpack
import pkg_resources


class CharConverter:

    def __init__(self, option='v2s'):
        """
        Initialize the converter with a option format.

        :param option: option format, either 'v2s' (simplified) or 'v2t' (traditional).
        :param mode: conversion mode, either 'one2one' or 'one2many'.
        """
        if option not in ['v2s', 'v2t']:
            raise ValueError("Option must be either 'v2s' (simplified) or 'v2t' (traditional).")
        mapping_file_path = pkg_resources.resource_filename('char_converter', f'data/{option}.msgpack')

        with open(mapping_file_path, 'rb') as f:
            self.mapping = msgpack.unpack(f, raw=False)
        
        self.mode = 'one2one'
    

    def set_mode(self, mode):
        """
        Set the conversion mode.

        :param mode: conversion mode, either 'one2one' or 'one2many'.
        """
        if mode not in ['one2one', 'one2many']:
            raise ValueError("Mode must be either 'one2one' or 'one2many'.")
        self.mode = mode
        print(f"Conversion mode is set to '{mode}'.")


    def convert(self, text):
        """
        Convert the text based on the mapping.

        :param text: The input text.
        :return: Converted text.
        """
        if self.mode == 'one2one':
            converted_text = [self.mapping.get(char, [char])[0] for char in text]
        elif self.mode == 'one2many':
            converted_text = [
                '【' + '|'.join(self.mapping.get(char, char)) + '】' if len(self.mapping.get(char, char)) > 1 
                else self.mapping.get(char, [char])[0] 
                for char in text
            ]
        return ''.join(converted_text)


    def convert_file(self, input_file, output_file):
        """
        Convert the text from an input file and save the converted text to an output file.

        :param input_file: Path to the input file.
        :param output_file: Path to the output file.
        """
        with open(input_file, 'r', encoding='utf-8-sig') as f_input:
            input_text = f_input.read()

        converted_text = self.convert(input_text)

        with open(output_file, 'w', encoding='utf-8-sig') as f_output:
            f_output.write(converted_text)


from setuptools import setup, find_packages

setup(
    name='char_converter',
    version='0.39',
    author='Yuqi Chen',
    author_email='cyq0722@pku.edu.cn',
    packages=find_packages(),
    install_requires=[
        'msgpack',
        'setuptools',
    ],
    package_data={
        'char_converter': ['*.py', 'data/*.msgpack']
    },
    include_package_data=True,
    description='A library to convert variant Chinese characters to standard simplified or traditional characters.',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
)
